# @file test_edk2_setup.py
# This contains unit tests for the edk2_ci_setup
##
# Copyright (c) Microsoft Corporation
#
# SPDX-License-Identifier: BSD-2-Clause-Patent
##
import pytest
import git
import pathlib
import sys
import logging

from edk2toolext.invocables import edk2_setup
from edk2toolext.environment import shell_environment

MIN_BUILD_FILE = r"""
from edk2toolext.invocables.edk2_setup import SetupSettingsManager, RequiredSubmodule
import pathlib

class Settings(SetupSettingsManager):
    # MIN BUILD FILE
    def GetWorkspaceRoot(self) -> str:
        return str(pathlib.Path(__file__).parent)

    def GetRequiredSubmodules(self) -> list[RequiredSubmodule]:
        return [
            RequiredSubmodule('Common/MU', True)
        ]

    def GetPackagesSupported(self) -> list[str]:
        return []

    def GetArchitecturesSupported(self) -> list[str]:
        return []

    def GetTargetsSupported(self) -> list[str]:
        return []
"""


EMPTY_BUILD_FILE = r"""
from edk2toolext.invocables.edk2_setup import SetupSettingsManager, RequiredSubmodule
import pathlib

class Settings(SetupSettingsManager):
    # EMPTY BUILD FILE
    def GetWorkspaceRoot(self) -> str:
        return str(pathlib.Path(__file__).parent)

    def GetRequiredSubmodules(self) -> list[RequiredSubmodule]:
        return []

    def GetPackagesSupported(self) -> list[str]:
        return []

    def GetArchitecturesSupported(self) -> list[str]:
        return []

    def GetTargetsSupported(self) -> list[str]:
        return []
"""


INVALID_REPO_BUILD_FILE = r"""
from edk2toolext.invocables.edk2_setup import SetupSettingsManager, RequiredSubmodule
import pathlib

class Settings(SetupSettingsManager):
    # INVALID REPO BUILD FILE
    def GetWorkspaceRoot(self) -> str:
        return str(pathlib.Path(__file__).parent.parent)

    def GetRequiredSubmodules(self) -> list[RequiredSubmodule]:
        return []

    def GetPackagesSupported(self) -> list[str]:
        return []

    def GetArchitecturesSupported(self) -> list[str]:
        return []

    def GetTargetsSupported(self) -> list[str]:
        return []
"""


INVALID_SUBMODULE_BUILD_FILE = r"""
from edk2toolext.invocables.edk2_setup import SetupSettingsManager, RequiredSubmodule
import pathlib

class Settings(SetupSettingsManager):
    # INVALID SUBMODULE BUILD FILE
    def GetWorkspaceRoot(self) -> str:
        return str(pathlib.Path(__file__).parent)

    def GetRequiredSubmodules(self) -> list[RequiredSubmodule]:
        return [
            RequiredSubmodule('Platforms', True)
        ]

    def GetPackagesSupported(self) -> list[str]:
        return []

    def GetArchitecturesSupported(self) -> list[str]:
        return []

    def GetTargetsSupported(self) -> list[str]:
        return []
"""


@pytest.fixture(scope="function")
def tree(tmpdir):
    """A fixture that provides a temporary directory containing the mu_tiano_platforms repo."""
    git.Repo.clone_from("https://github.com/microsoft/mu_tiano_platforms.git", tmpdir)
    tmpdir = pathlib.Path(tmpdir)
    return pathlib.Path(tmpdir)


def write_build_file(tree, file):
    """Writes the requested build file to the base of the tree."""
    build_file = tree / "BuildFile.py"
    with open(build_file, 'x') as f:
        f.write(file)
    return build_file


def test_setup_simple_repo(tree: pathlib.Path):
    """Tests that edk2_setup can successfuly clone a submodule."""
    #############################################
    # Perform Initial Setup and verify it works #
    #############################################
    min_build_file = write_build_file(tree, MIN_BUILD_FILE)
    sys.argv = ["stuart_setup", "-c", str(min_build_file), "-v"]
    mu_submodule = tree / "Common" / "MU"

    assert len(list(mu_submodule.iterdir())) == 0  # The MU submodule should not exist
    try:
        edk2_setup.main()
    except SystemExit as e:
        assert e.code == 0
    assert len(list(mu_submodule.iterdir())) > 0  # The MU submodule should exist

    #######################################################################
    # Force mu_tiano_platforms clean (The build file we wrote dirtied it) #
    #######################################################################
    with git.Repo(tree) as repo:
        assert repo.is_dirty(untracked_files=True) is True

    sys.argv = ["stuart_setup", "-c", str(min_build_file), "-v", "--FORCE"]
    try:
        edk2_setup.main()
    except SystemExit as e:
        assert e.code == 0

    with git.Repo(tree) as repo:
        assert repo.is_dirty(untracked_files=True) is False

    #############################################################
    # Dirty a submodule file and verify we skip without --FORCE #
    #############################################################
    min_build_file = write_build_file(tree, MIN_BUILD_FILE)
    with open(mu_submodule / "License.txt", 'a') as f:
        f.write("TEST")

    with git.Repo(mu_submodule) as repo:
        assert repo.is_dirty(untracked_files=True) is True

    sys.argv = ["stuart_setup", "-c", str(min_build_file), "-v"]
    try:
        edk2_setup.main()
    except SystemExit as e:
        assert e.code == 0

    with git.Repo(mu_submodule) as repo:
        assert repo.is_dirty(untracked_files=True) is True

    #############################################################
    # Verify we can clean the dirty submodule with --FORCE      #
    #############################################################
    with git.Repo(mu_submodule) as repo:
        assert repo.is_dirty(untracked_files=True) is True

    sys.argv = ["stuart_setup", "-c", str(min_build_file), "-v", "--FORCE"]
    try:
        edk2_setup.main()
    except SystemExit as e:
        assert e.code == 0

    with git.Repo(mu_submodule) as repo:
        assert repo.is_dirty(untracked_files=True) is False


def test_setup_bad_omnicache(caplog, tree: pathlib.Path):
    """Tests that edk2_setup catches a bad omnicache path."""
    caplog.at_level(logging.WARNING)  # Capture only warnings
    empty_build_file = write_build_file(tree, EMPTY_BUILD_FILE)
    sys.argv = ["stuart_setup", "-c", str(empty_build_file), "-v", "--omnicache", "does_not_exist"]

    try:
        edk2_setup.main()
    except SystemExit as e:
        assert e.code == 0

    # Verify we output a warning about the omnicache path being invalid
    for record in caplog.records:
        if "Omnicache path set to invalid path" in record.msg:
            break
    else:
        pytest.fail("Did not find a warning about the omnicache path being invalid.")


def test_setup_invalid_repo(tree: pathlib.Path):
    """Tests that edk2_setup catches a bad omnicache path."""
    invalid_build_file = write_build_file(tree, INVALID_REPO_BUILD_FILE)
    sys.argv = ["stuart_setup", "-c", str(invalid_build_file), "-v", "--FORCE"]
    try:
        edk2_setup.main()
    except SystemExit as e:
        assert e.code == -1


def test_setup_invalid_submodule(tree: pathlib.Path):
    """Tests that edk2_setup catches a bad submodule path."""
    invalid_build_file = write_build_file(tree, INVALID_SUBMODULE_BUILD_FILE)
    sys.argv = ["stuart_setup", "-c", str(invalid_build_file), "-v"]
    try:
        edk2_setup.main()
    except SystemExit as e:
        assert e.code == -1

    sys.argv = ["stuart_setup", "-c", str(invalid_build_file), "-v", "--FORCE"]
    try:
        edk2_setup.main()
    except SystemExit as e:
        assert e.code == -1


def test_parse_command_line_options(tree: pathlib.Path):
    """Tests that the command line parser works correctly."""
    # Test valid command line options
    empty_build_file = write_build_file(tree, EMPTY_BUILD_FILE)
    sys.argv = [
        "stuart_setup", "-c", str(empty_build_file),
        "BLD_*_VAR",
        "VAR",
        "BLD_DEBUG_VAR2",
        "BLD_RELEASE_VAR2",
        "TEST_VAR=TEST",
        "BLD_*_TEST_VAR2=TEST"
    ]
    try:
        edk2_setup.main()
    except SystemExit as e:
        assert e.code == 0

    env = shell_environment.GetBuildVars()
    assert env.GetValue("BLD_*_VAR") is not None
    assert env.GetValue("VAR") is not None
    assert env.GetValue("BLD_DEBUG_VAR2") is not None
    assert env.GetValue("BLD_RELEASE_VAR2") is not None
    assert env.GetValue("TEST_VAR") == "TEST"
    assert env.GetValue("BLD_*_TEST_VAR2") == "TEST"

    # Test invalid command line options
    for arg in ["BLD_*_VAR=5=10", "BLD_DEBUG_VAR2=5=5", "BLD_RELEASE_VAR3=5=5", "VAR=10=10"]:
        sys.argv = [
            "stuart_setup",
            "-c", str(empty_build_file),
            arg
        ]
        try:
            edk2_setup.main()
        except RuntimeError as e:
            assert str(e).startswith(f"Unknown variable passed in via CLI: {arg}")


def test_conf_file(tree: pathlib.Path):
    """Tests that the config file parser works correctly."""
    empty_build_file = write_build_file(tree, EMPTY_BUILD_FILE)
    build_conf = tree / 'BuildConfig.conf'
    with open(build_conf, 'x') as f:
        f.writelines([
            "BLD_*_VAR",
            "\nVAR",
            "\nBLD_DEBUG_VAR2",
            "\nBLD_RELEASE_VAR2",
            "\nTEST_VAR=TEST",
            "\nBLD_*_TEST_VAR2=TEST"
        ])
    sys.argv = ["stuart_setup", "-c", str(empty_build_file)]

    try:
        edk2_setup.main()
    except SystemExit as e:
        assert e.code == 0

    env = shell_environment.GetBuildVars()
    assert env.GetValue("BLD_*_VAR") is not None
    assert env.GetValue("VAR") is not None
    assert env.GetValue("BLD_DEBUG_VAR2") is not None
    assert env.GetValue("BLD_RELEASE_VAR2") is not None
    assert env.GetValue("TEST_VAR") == "TEST"
    assert env.GetValue("BLD_*_TEST_VAR2") == "TEST"

    # Test invalid build config
    for arg in ["BLD_*_VAR=5=10", "BLD_DEBUG_VAR2=5=5", "BLD_RELEASE_VAR3=5=5", "VAR=10=10"]:
        build_conf.unlink()
        with open(build_conf, 'x') as f:
            f.writelines([arg])

        sys.argv = [
            "stuart_setup",
            "-c", str(empty_build_file),
            arg
        ]
        try:
            edk2_setup.main()
        except RuntimeError as e:
            assert str(e).startswith(f"Unknown variable passed in via CLI: {arg}")



# @file test_repo_resolver.py
# This contains unit tests for repo resolver
#
##
# Copyright (c) Microsoft Corporation
#
# SPDX-License-Identifier: BSD-2-Clause-Patent
##
import logging
import os
import unittest
from edk2toolext.environment import repo_resolver
from edk2toollib.utility_functions import RemoveTree
import tempfile
import pathlib
import pytest


branch_dependency = {
    "Url": "https://github.com/microsoft/mu",
    "Path": "test_repo",
    "Branch": "master"
}

sub_branch_dependency = {
    "Url": "https://github.com/microsoft/mu",
    "Path": "test_repo",
    "Branch": "gh-pages"
}

commit_dependency = {
    "Url": "https://github.com/microsoft/mu",
    "Path": "test_repo",
    "Commit": "b1e35a5d2bf05fb7f58f5b641a702c70d6b32a98"
}

short_commit_dependency = {
    "Url": "https://github.com/microsoft/mu",
    "Path": "test_repo",
    "Commit": "b1e35a5"
}

commit_later_dependency = {
    "Url": "https://github.com/microsoft/mu",
    "Path": "test_repo",
    "Commit": "e28910950c52256eb620e35d111945cdf5d002d1"
}

microsoft_commit_dependency = {
    "Url": "https://github.com/Microsoft/microsoft.github.io",
    "Path": "test_repo",
    "Commit": "e9153e69c82068b45609359f86554a93569d76f1"
}
microsoft_branch_dependency = {
    "Url": "https://github.com/Microsoft/microsoft.github.io",
    "Path": "test_repo",
    "Commit": "e9153e69c82068b45609359f86554a93569d76f1"
}

test_dir = None


def prep_workspace():
    global test_dir
    # if test temp dir doesn't exist
    if test_dir is None or not os.path.isdir(test_dir):
        test_dir = tempfile.mkdtemp()
        logging.debug("temp dir is: %s" % test_dir)
    else:
        repo_resolver.clear_folder(test_dir)
        test_dir = tempfile.mkdtemp()


def clean_workspace():
    global test_dir
    if test_dir is None:
        return

    if os.path.isdir(test_dir):
        repo_resolver.clear_folder(test_dir)
        test_dir = None


def get_first_file(folder):
    folder_list = os.listdir(folder)
    for file_path in folder_list:
        path = os.path.join(folder, file_path)
        if os.path.isfile(path):
            return path
    return None


class test_repo_resolver(unittest.TestCase):
    def setUp(self):
        prep_workspace()

    @classmethod
    def setUpClass(cls):
        logger = logging.getLogger('')
        logger.setLevel(logging.DEBUG)
        logger.addHandler(logging.NullHandler())
        unittest.installHandler()

    @classmethod
    def tearDown(cls):
        clean_workspace()

    # check to make sure that we can clone a branch correctly
    def test_clone_branch_repo(self):
        # create an empty directory- and set that as the workspace
        repo_resolver.resolve(test_dir, branch_dependency)
        folder_path = os.path.join(test_dir, branch_dependency["Path"])
        details = repo_resolver.repo_details(folder_path)
        self.assertEqual(details["Url"], branch_dependency['Url'])
        self.assertEqual(details["Branch"], branch_dependency['Branch'])

    def test_clone_branch_existing_folder(self):
        # Resolve when folder exists but is empty
        folder_path = os.path.join(test_dir, branch_dependency["Path"])
        os.makedirs(folder_path)
        repo_resolver.resolve(test_dir, branch_dependency)
        details = repo_resolver.repo_details(folder_path)
        self.assertEqual(details["Url"], branch_dependency['Url'])
        self.assertEqual(details["Branch"], branch_dependency['Branch'])

    # don't create a git repo, create the folder, add a file, try to clone in the folder, it should throw an exception
    def test_wont_delete_files(self):
        folder_path = os.path.join(test_dir, commit_dependency["Path"])
        os.makedirs(folder_path)
        file_path = os.path.join(folder_path, "test.txt")
        file_path = os.path.join(
            test_dir, branch_dependency["Path"], "test.txt")
        out_file = open(file_path, "w+")
        out_file.write("Make sure we don't delete this")
        out_file.close()
        self.assertTrue(os.path.isfile(file_path))
        with self.assertRaises(Exception):
            repo_resolver.resolve(test_dir, branch_dependency)
            self.fail("We shouldn't make it here")
        self.assertTrue(os.path.isfile(file_path))

    # don't create a git repo, create the folder, add a file, try to clone in the folder, will force it to happen
    def test_will_delete_files(self):
        folder_path = os.path.join(test_dir, commit_dependency["Path"])
        os.makedirs(folder_path)
        file_path = os.path.join(folder_path, "test.txt")
        out_file = open(file_path, "w+")
        out_file.write("Make sure we don't delete this")
        out_file.close()
        self.assertTrue(os.path.exists(file_path))
        try:
            repo_resolver.resolve(test_dir, commit_dependency, force=True)
        except:
            self.fail("We shouldn't fail when we are forcing")
        details = repo_resolver.repo_details(folder_path)
        self.assertEqual(details["Url"], commit_dependency['Url'])

    # don't create a git repo, create the folder, add a file, try to clone in the folder, will ignore it.
    def test_will_ignore_files(self):
        folder_path = os.path.join(test_dir, commit_dependency["Path"])
        os.makedirs(folder_path)
        file_path = os.path.join(folder_path, "test.txt")
        out_file = open(file_path, "w+")
        out_file.write("Make sure we don't delete this")
        out_file.close()
        self.assertTrue(os.path.exists(file_path))
        repo_resolver.resolve(test_dir, commit_dependency, ignore=True)

    def test_wont_delete_dirty_repo(self):
        repo_resolver.resolve(test_dir, commit_dependency)

        folder_path = os.path.join(test_dir, commit_dependency["Path"])
        file_path = get_first_file(folder_path)
        # make sure the file already exists
        self.assertTrue(os.path.isfile(file_path))
        out_file = open(file_path, "a+")
        out_file.write("Make sure we don't delete this")
        out_file.close()
        self.assertTrue(os.path.exists(file_path))

        with self.assertRaises(Exception):
            repo_resolver.resolve(test_dir, commit_dependency, update_ok=True)

        # Get passed this with ignore=true
        repo_resolver.resolve(test_dir, commit_dependency, ignore=True)

    def test_will_delete_dirty_repo(self):
        repo_resolver.resolve(test_dir, commit_dependency)
        folder_path = os.path.join(test_dir, commit_dependency["Path"])
        file_path = get_first_file(folder_path)
        # make sure the file already exists
        self.assertTrue(os.path.isfile(file_path))
        out_file = open(file_path, "a+")
        out_file.write("Make sure we don't delete this")
        out_file.close()
        self.assertTrue(os.path.exists(file_path))

        try:
            repo_resolver.resolve(test_dir, commit_later_dependency, force=True)
        except:
            self.fail("We shouldn't fail when we are forcing")

    # check to make sure we can clone a commit correctly
    def test_clone_commit_repo(self):
        # create an empty directory- and set that as the workspace
        repo_resolver.resolve(test_dir, commit_dependency)
        folder_path = os.path.join(test_dir, commit_dependency["Path"])
        details = repo_resolver.repo_details(folder_path)

        self.assertEqual(details["Url"], commit_dependency['Url'])
        self.assertEqual(details["Head"]["HexSha"], commit_dependency['Commit'])

    # check to make sure we support short commits
    def test_clone_short_commit_repo(self):
        repo_resolver.resolve(test_dir, short_commit_dependency)
        folder_path = os.path.join(test_dir, short_commit_dependency["Path"])
        details = repo_resolver.repo_details(folder_path)

        self.assertEqual(details["Url"], short_commit_dependency['Url'])
        self.assertEqual(details["Head"]["HexShaShort"], short_commit_dependency['Commit'])
        # Resolve again, making sure we don't fail if repo already exists.
        repo_resolver.resolve(test_dir, short_commit_dependency)

    # check to make sure we can clone a commit correctly
    def test_fail_update(self):
        # create an empty directory- and set that as the workspace
        repo_resolver.resolve(test_dir, commit_dependency)
        folder_path = os.path.join(test_dir, commit_dependency["Path"])
        details = repo_resolver.repo_details(folder_path)

        self.assertEqual(details["Url"], commit_dependency['Url'])
        self.assertEqual(details["Head"]["HexSha"], commit_dependency['Commit'])
        # first we checkout
        with self.assertRaises(Exception):
            repo_resolver.resolve(test_dir, commit_later_dependency)

        details = repo_resolver.repo_details(folder_path)
        self.assertEqual(details["Url"], commit_dependency['Url'])
        self.assertEqual(details["Head"]["HexSha"], commit_dependency['Commit'])

    def test_does_update(self):
        # create an empty directory- and set that as the workspace
        logging.info(f"Resolving at {test_dir}")
        repo_resolver.resolve(test_dir, commit_dependency)
        folder_path = os.path.join(test_dir, commit_dependency["Path"])
        logging.info(f"Getting details at {folder_path}")
        details = repo_resolver.repo_details(folder_path)

        self.assertEqual(details["Url"], commit_dependency['Url'])
        self.assertEqual(details["Head"]["HexSha"], commit_dependency['Commit'])
        # next we checkout and go to the later commit
        try:
            repo_resolver.resolve(
                test_dir, commit_later_dependency, update_ok=True)
        except:
            self.fail("We are not supposed to throw an exception")
        details = repo_resolver.repo_details(folder_path)
        logging.info(f"Checking {folder_path} for current git commit")

        self.assertEqual(details["Url"], commit_later_dependency['Url'])
        self.assertEqual(details["Head"]["HexSha"], commit_later_dependency['Commit'])

    def test_cant_switch_urls(self):
        # create an empty directory- and set that as the workspace
        repo_resolver.resolve(test_dir, branch_dependency)
        folder_path = os.path.join(test_dir, branch_dependency["Path"])

        details = repo_resolver.repo_details(folder_path)

        self.assertEqual(details["Url"], branch_dependency['Url'])
        # first we checkout
        with self.assertRaises(Exception):
            repo_resolver.resolve(test_dir, microsoft_branch_dependency)

        details = repo_resolver.repo_details(folder_path)
        self.assertEqual(details["Url"], branch_dependency['Url'])

    def test_ignore(self):
        # create an empty directory- and set that as the workspace
        repo_resolver.resolve(test_dir, branch_dependency)
        folder_path = os.path.join(test_dir, branch_dependency["Path"])

        details = repo_resolver.repo_details(folder_path)

        self.assertEqual(details["Url"], branch_dependency['Url'])
        # first we checkout

        repo_resolver.resolve(
            test_dir, microsoft_branch_dependency, ignore=True)

        details = repo_resolver.repo_details(folder_path)
        self.assertEqual(details["Url"], branch_dependency['Url'])

    def test_will_switch_urls(self):
        # create an empty directory- and set that as the workspace
        repo_resolver.resolve(test_dir, branch_dependency)

        folder_path = os.path.join(test_dir, branch_dependency["Path"])

        details = repo_resolver.repo_details(folder_path)

        self.assertEqual(details["Url"], branch_dependency['Url'])
        # first we checkout
        try:
            repo_resolver.resolve(
                test_dir, microsoft_branch_dependency, force=True)
        except:
            self.fail("We shouldn't fail when we are forcing")

        details = repo_resolver.repo_details(folder_path)
        self.assertEqual(details["Url"], microsoft_branch_dependency['Url'])

    def test_will_switch_branches(self):
        repo_resolver.resolve(test_dir, branch_dependency)
        folder_path = os.path.join(test_dir, branch_dependency["Path"])
        repo_resolver.resolve(test_dir, sub_branch_dependency, force=True)
        details = repo_resolver.repo_details(folder_path)
        self.assertEqual(details["Url"], branch_dependency['Url'])
        self.assertEqual(details["Branch"], sub_branch_dependency['Branch'])

    def test_submodule(self):

        class Submodule():
            def __init__(self, path, recursive):
                self.path = path
                self.recursive = recursive

        temp_folder = tempfile.mkdtemp()
        submodule_path = "Common/MU_TIANO"
        deps = {"Url": "https://github.com/microsoft/mu_tiano_platforms"}
        repo_resolver.clone_repo(temp_folder, deps)

        os.mkdir(os.path.join(temp_folder, "Build"))
        tmp_file = os.path.join(temp_folder, "Build", "tempfile.txt")
        with open(tmp_file, 'x') as f:
            f.write("Temp edit")
            self.assertTrue(os.path.isfile(tmp_file))

            repo_resolver.clean(temp_folder, ignore_files=["Build/tempfile.txt"])
            self.assertTrue(os.path.isfile(tmp_file))

        repo_resolver.clean(temp_folder)
        self.assertFalse(os.path.isfile(tmp_file))

        repo_resolver.submodule_resolve(temp_folder, Submodule(submodule_path, True))

        RemoveTree(temp_folder)


def test_resolve_all(tmpdir: pathlib.Path):
    deps = [
        {
            "Url": "https://github.com/octocat/Spoon-Knife",
            "Path": "repo1",
            "Commit": "a30c19e3f13765a3b48829788bc1cb8b4e95cee4"
        },
        {
            "Url": "https://github.com/octocat/Spoon-Knife",
            "Path": "repo2",
            "Commit": "bb4cc8d3b2e14b3af5df699876dd4ff3acd00b7f"
        },
        {
            "Url": "https://github.com/octocat/Spoon-Knife",
            "Path": "repo3",
            "Commit": "d0dd1f61b33d64e29d8bc1372a94ef6a2fee76a9"
        },
    ]

    repos = repo_resolver.resolve_all(tmpdir, deps, force=True)

    for repo in repos:
        assert len(list(pathlib.Path(repo).iterdir())) > 0


def test_clone_from_with_reference(tmpdir: pathlib.Path):
    # Clone the reference repo
    ref_path = pathlib.Path(tmpdir / "ref")
    dep = {"Url": "https://github.com/octocat/Spoon-Knife"}
    repo_resolver.clone_repo(ref_path, dep)
    assert len(list(ref_path.iterdir())) > 0

    # Clone the repo from the reference
    repo_path1 = pathlib.Path(tmpdir / "repo1")
    dep = {"Url": "https://github.com/octocat/Spoon-Knife", "ReferencePath": ref_path}
    repo_resolver.clone_repo(repo_path1, dep)
    assert len(list(repo_path1.iterdir())) > 0

    # Clone the repo from a bad reference
    repo_path2 = pathlib.Path(tmpdir / "repo2")
    bad_repo_path = pathlib.Path(tmpdir / "bad_repo")
    dep = {"Url": "https://github.com/octocat/Spoon-Knife", "ReferencePath": bad_repo_path}
    repo_resolver.clone_repo(repo_path2, dep)
    assert len(list(repo_path2.iterdir())) > 0


def test_checkout_branch(tmpdir: pathlib.Path):
    # Clone the repo
    dep = {"Url": "https://github.com/octocat/Spoon-Knife", "Path": tmpdir, "Branch": "main"}
    repo_resolver.clone_repo(tmpdir, dep)
    details = repo_resolver.repo_details(tmpdir)
    assert details["Branch"] == "main"

    # Checkout same branch
    repo_resolver.checkout(tmpdir, dep)
    details = repo_resolver.repo_details(tmpdir)
    assert details["Branch"] == "main"

    # Dep for the rest of the tests
    dep = {"Url": "https://github.com/octocat/Spoon-Knife", "Path": tmpdir, "Branch": "test-branch"}

    # Checkout different branch with ignore_dep_state_mismatch
    repo_resolver.checkout(tmpdir, dep, ignore_dep_state_mismatch=True)
    details = repo_resolver.repo_details(tmpdir)
    assert details["Branch"] == "main"

    # Checkout different branch without force
    with pytest.raises(Exception):
        repo_resolver.checkout(tmpdir, dep)

    # Checkout different branch with force
    repo_resolver.checkout(tmpdir, dep, force=True)
    details = repo_resolver.repo_details(tmpdir)
    assert details["Branch"] == "test-branch"


def test_checkout_bad_branch(tmpdir: pathlib.Path):
    # Clone the repo
    dep = {"Url": "https://github.com/octocat/Spoon-Knife", "Path": tmpdir, "Branch": "main"}
    repo_resolver.clone_repo(tmpdir, dep)
    details = repo_resolver.repo_details(tmpdir)
    assert details["Branch"] == "main"

    # Checkout bad branch without forcing
    dep = {"Url": "https://github.com/octocat/Spoon-Knife", "Path": tmpdir, "Branch": "does not exist"}
    with pytest.raises(Exception):
        repo_resolver.checkout(tmpdir, dep)

    # Checkout bad branch with forcing
    with pytest.raises(repo_resolver.GitCommandError):
        repo_resolver.checkout(tmpdir, dep, force=True)



# @file test_pyopenssl_signer.py
# This contains unit tests for the pyopenssl binary wrapper
##
# Copyright (c) Microsoft Corporation
#
# SPDX-License-Identifier: BSD-2-Clause-Patent
##
import unittest
from base64 import b64decode
from edk2toolext.capsule import pyopenssl_signer

r"""
TESTCERT is a cert used to test the ability to parse pfx and pull out necessary information.
To generate a new test cert follow these steps:

1. Generate the Cert (.pfx) file
There are multiple ways to do this, but running this powershell script is the easiest:

``` powershell
$myPwd = ConvertTo-SecureString "password" -Force -AsPlainText
$Cert = New-SelfSignedCertificate `
    -DnsName "www.contoso.com" `
    -CertStoreLocation "Cert:\LocalMachine\My\" `
    -KeyAlgorithm RSA `
    -KeyLength 2048 `
    -Subject "CN=Mock Platform Key" `
    -NotAfter (Get-Date).AddYears(10)
Export-PfxCertificate `
    -Cert ("Cert:\LocalMachine\My\"+$Cert.Thumbprint) `
    -FilePath ./Certificate.pfx `
    -Password $myPwd
```

2. Serialize the Cert
Run the following Python commands to read the cert and print the bytes in a
format allowing it to be used in a python file while meeting flake8 requirements:
``` python
from base64 import b64encode
import textwrap
with open("path/to/cert.pfx", 'rb') as file:
    gfg = b64encode(file.read())
    lines = textwrap.wrap(str(gfg), 60, initial_indent="TESTCERT1 = ")
    for t in range(len(lines)):
        end = "\\\n" if t + 1 < len(lines) else "\n"
        print(lines[t], end=end)
```
From there, you can copy the console printed serialized cert and use it in your
python file as seen below this docstring.
"""

# spell-checker:disable
# TESTCERT1 has no password
TESTCERT1 = b'MIIJQQIBAzCCCQcGCSqGSIb3DQEHAaCCCPgEggj0MIII8D\
CCA6cGCSqGSIb3DQEHBqCCA5gwggOUAgEAMIIDjQYJKoZIhvcNAQcBMBwGCi\
qGSIb3DQEMAQYwDgQIlimkkLBfTQ8CAggAgIIDYAGQRMWf8SbXd6nZPL2o11\
LF1JrXLCCc+RwJCDnAJD8cNFUwq4JrOc4qJOYr6QZwD//3LfHeNLwfsi+3RC\
mQxiRO0ZH/19QS0/gvobJSdKgvE1ywkB1w5h9uD0R1HQHInlTqUCtC36QhG/\
bjBCmthcSNzkBoVMi/fY9fU2Ldg76ivk7Wn9a/EHorc5XgH2Lg4k4WCIB9rS\
p2m2/AVJ2aejS5fZHOgRXOb5YeO3TyyG36bsAy6Le7ZqcLEtESd4CbjCKJ1i\
mcGdFk18Jh3JN/lqefpB3W3l9YRxL2Cu8jzarlpWQw6O7Q2CjLbZ5T+kSZqK\
EByQLViCvCILYyTB8Kr5go/GGuU2oZYC1dWprxGDhznMO/KewNXec42h7j6K\
rIS6ftJFMLKYBsfsZ8Q3MIvCcPNz/ch7lymzibtjYJE8VwJlcL/RP17tOi4J\
hC44gdErhUjj87s3kh6ZDY3FToU7u/voQd/1ipRTZCurOFc20RbhDfyOyafv\
SyASl7Hlk3+sGVGHvtrSBoSYsW+4XNyOfqnZhp+EzrBmg9AjfvJniWtMTbWI\
ekO/DbjVxXww+/AXkTgqcutjFerTWj16fKy7wqM3tCVQiTsbX7uiWDF6nEF6\
NQAZ1EsUaEw2B45eUh8xtML3j70zFQRwIJ+qnPxuj3ouWk4ShZKjQD39EsJn\
5aVzGWVIF9tPgJo10E1vf+doIHmpgpzYHJi+fLPJZ6bbrzpntK9vcnUjnsK1\
yRax/+381jhIZbAG54ruOnANfUhOWijObTdE5we2g9j3JpzXWUYoFUBFqBx4\
hYAwgFw//fp3b8VtI8xUnzrvfBCF4rUyykWT7YnUM8VuBFgq5GGPUt1PZaw/\
jemPMV5WNe+ReL/BKjniZH5ZwITbz4PHm6Qt/G5tM/h46YgRux5jy7KriPQw\
PqgoJK8JFs8I/6mQkv2fNY6ttbRMbSdi0yLPpqgQk8EEJV8mvI/bwXGeJTLQ\
JhXCau48DBR9nYcyaYCDX3z84yZpVJFRJyWT9GvTMbA3zbS3BdO1oaC/60No\
1faabkgJyzXeNmY/v293JI0sSLB51tQr2amSH1mu2kvvhMegs2qw57Kg8r3i\
W9tjo7GCepbUbXa9BE09ZDRsuOuNWfs/2Pe6X6P2Hg3gc7+aaspDCCBUEGCS\
qGSIb3DQEHAaCCBTIEggUuMIIFKjCCBSYGCyqGSIb3DQEMCgECoIIE7jCCBO\
owHAYKKoZIhvcNAQwBAzAOBAjHl66YnVMBEwICCAAEggTIpiydhQsoj1647s\
pVJEC07g1PcNCGS985FvjILNgvEm8uZe2qPWK5wVhN81vQ6lx65q7DXqEncx\
2UdzidAxgDxVAHBBqHhLXkDMItng9/Q4rVS9E5IPsOVrtl+0tXsYvcktE/1T\
dVoNzClYNrwBE82Nc9E+fDBEQ6MLF7pnF1IgfVgMOHOkn4GT8XgZ9Y8kg6us\
/PpexIQwayRnIa5hmVAlVKYUuW9WZR782PDFPU+DwwIe+LMwmDubGxWDqBID\
hhPG9vhdWqb7U5wl33ryLAs1/SMC1J9AbOUQIJvTJfnFd/hfg35lq6lCiEFZ\
S+dPdAXqVOOgdHlusTskeYlol5EZxPATDYHm5t7PaiQ5PtfRQN1G2T8yB/eo\
/NbA4l1vzscTHybqucb8AAqf0WdsPXo+VT4FpK8SBM/C5TANRbuxEm9rNnKN\
I0VIZV+4etGHmi8OW5y65SysUn/D7RL1atHEEPcMesfWiks7aBZLwYJCpO5p\
Ry5n7T3Ca/E19GSubnYSsXAmXfBJ+lw8jq99yjPML+jM/lGzv300ajpzjDtI\
xyTfbaO0IH081RfDmmer1dkd5C1RaOzswRIga7KnXjyJz/f4Yh4tCdKsQN44\
PE6ib+dvZWED9vX7Zh8IqdM7bdZB6Lj8ZDQD2nZ/7rkOJpqR6qisTaAUdnmw\
DN0UJ9F8OxEkS6YodI9a/xjmtxFGbDVCow0R01zBiK+letgrr5iPnXEadiFd\
+E8zcCl5Qkk+gP8PqphOGLZZgEkkeLETk82w9D0XuBHc2grXlcNUvRBm1D8j\
VAIIrIogvNJL/itjHAoZEjeE8n0RQYmN2cWMJlNk0Ifhc17Ycv9O09QO4RY6\
OOMRaFtRSOJScrxTPVHSyceSWs85HV11BYzYYZQd0xHy1+nz2ZKyrm7VXsWf\
qIWE/HkyspodjUUNBV2iPJR6UHxloS5EGpluG+ApyXMh/efjRZ9isLUp9lEA\
n8HghTJwi/vpBwWot4nGrg6RpHKrmhxAi564rVHuJrhDCqQvAx1atO8zmCqJ\
Ppt7kYm3ll6k4HZPetlQzqfGLedS4a4OHb+3c4VaqONUTZDAE0cQ4KeR/byf\
Z+UIDGAosmEWk49CnrPF/eD3Ul79yBNqOZIg3P6V+pwek9nYX8fCWWRk4Ayc\
SQjHY0WdTCUUvc14TwO5Lc/AZSrnCG5KB/YuIhgQlLKMRioQ4uS829LNYAhz\
8xSRG9MRZjxIdUevqxDEI1/eKk2+qBnq+qVqoltPw+8mqnsMM3qRMtBZydLV\
s5pMklXuIMGVe2PGLwVY2cbc4ms4m2PpyLBO0DqlZJ5G8hJuq11SE8jjfI3K\
O9xc6D28mryoSTJE3ruN3mlRdbiAsNg15mUdXq9M9AthLaeSyCLT3a2QP1zt\
O+AHsgd6ovMhfxAZ1R/0YayjtKB6Z4CN5rEAg2QdR5nvb6G4EKyqWAZhCCcd\
UezFsMna+YaA/F2o9WIj9KPB+rdNh/KcbIJhpA2DSbbTPejHAk1TYBWall3B\
2CfsFEIMjtuid/ggz/pUYCb88PN3BPoQ7GO/jv6Vi6F4oCQz+Y0srbmIStNj\
qKXmc5OHIJoRkbrbFi4+2BeevncDADGrhl3heGWSmRlbOPxrryNrDPw7cbMS\
UwIwYJKoZIhvcNAQkVMRYEFILT9pNixD3s66GdK3I48b/dr23DMDEwITAJBg\
UrDgMCGgUABBTdiDC3a0y1gAbO1eZqveI3Zd0BrwQIFO8dSTjGLhMCAggA'

# TESTCERT2 has a password
TESTCERT2 = b'MIIKaQIBAzCCCiUGCSqGSIb3DQEHAaCCChYEggoSMIIKDj\
CCBg8GCSqGSIb3DQEHAaCCBgAEggX8MIIF+DCCBfQGCyqGSIb3DQEMCgECoI\
IE/jCCBPowHAYKKoZIhvcNAQwBAzAOBAh8sw/XNGIGZwICB9AEggTYeUQ8gJ\
GII6HPDeBx4bGH2zkS1ybw4hToh8c9UMd3gwDviCobjkSCidTGZPwkvcfPT8\
3WnZK0Er3fflYL3Y1BD/4Tzzoobstt+Afl7wKiEMWK95xXKe/zFD4ipKN0Z3\
LgpYo+KUvMd5QG0m0OfejLFn/pABkSLFxvHo4KFVHOboqaVZbnG9OftUR2tZ\
dFYaUk/aFSvJKA/M7M/2QI1Byj04bdwFr+1WzEag7kU/Wd/XVmRUTqq9ljmj\
d6xRWuk0NGq4j/OpRIFrJVn585wRlyRl8AGwBf8aIoWgJPK/67y26azde0G5\
YHVDO++sn07K+W2EOaD/UNf21NsD9fZYDyDPTxkKK/e3KjLiKWvK3KG5e7UB\
REKkizsUf7xkP6vRwifyAkC6LgM2UgFhcCYsdsBtMqcWZsjLj5hVg54bBD5Q\
qbcyC5ijUBjBj7ZXKsLDkmZY34VwzzXFcY+jMWhLEojZF9bmO8n58GI4bGhB\
LDL+TzDAiA9cK6VqxGECTW75Z9MkKdECPhfM1L1hTse7M70bo6aX1RwHBHUO\
RkdtEIXIsr1fRf47T83hzlAlxZkDbdtAitnYOUoXI3l+bupIWVpLIcyj+avK\
tdFTIHLmeJR22sy7DTYzS1fYxDlq3MwZmIdRj3LmarM6c4gmk0S/M4f+HvW6\
HyeKt+1cb0E4WlquvLKnRshwdvZ5xV8NL8waOsp4e0xQGPiusDnTspf5ozlc\
eTb3vdE6as0JyYb+YajYZY+3x5Bj4hA3GnQOKRvSQ9h5H7vH5HRQv6NIZrtt\
b9cl6y3R7hMo5uSW5vkWUkus0T6Q47c3SGTtSdqbR5p1+CphQNX6q3bEeQ+r\
fs14CLVtLVFcYoi4+/z33lvkaaZQWvFRHsvKtJIfKcaSJAkBrKXNAJ7nOqbJ\
vXajIcUzmmExTz+73LFj1tHil2RU7YGPqj+Mm0C+/N2xt6u5kD1OPMcdqOGj\
uJNFBdQPOxE3Okdo5zKhIXb12p7P2S3SP3FMZJWS3mEISmGU+4ugctDNINsZ\
YDr9BjPCC6XfRVLZ1VfzTVc9BmcxptcXdfw8U0SHaJ7tBvJfA94k5uo7QNGV\
XC6BLgNAOsimqtFGdtK4mO/wazA28Q/h2bj/KjyFRkXWg52qBSisqA07duwt\
csNOW5MOPWhYGF5ErBBJi1etsHyzqg0UyAN+4I+P8vqc5vnFFbesGOdmMmnU\
18m50yx3bxHU4E3oJF0kiVoAVnjfpDk6uCOf/raWE8W6Aby0jL4tQayX78UE\
G+wPFTBNr0zkX4AfQKvPLVn8PiDiKQ+w7+Mix6kQr3QlcxLVmcOmHKZdr7X/\
XwJlU/q/JNLeheucnd7uaFTHZqXN7Y2m7OlKGC6unCHryV35wjDZnhh1Ug0a\
E8lAejJ2F4vk9oYjHVv1NnwGZ5ZGnaTdasJkbB0LvsG5aQ6qXZ/SpuwK23Ph\
uAFmeW2HrP6UtzJhBAdRWsEYvEl4l9YuD9sC/C+pklAgo1lQB6omPETYE3EW\
VobsvxgJ60CmvLIPXDJMQt4FjrsdzfqnCxuN3F3NT5qnP6tD2Ru1RYcf+TNY\
cM95LW3BhG+FtoPKxU/ABFXCoN+tm0CS+JzB+5TafRbZtkO5iYwk4LXTaHJn\
8p6LF/sCEd1ny6veUJ5HswwzxbUDGB4jANBgkrBgEEAYI3EQIxADATBgkqhk\
iG9w0BCRUxBgQEAQAAADBdBgkqhkiG9w0BCRQxUB5OAHQAZQAtADQANwAzAD\
MAMABiADUAYQAtADEAMABlADUALQA0ADkAMgA4AC0AOQA5AGIANgAtADMAYw\
BkADgAMwBlADQAOAA2AGEAZgA1MF0GCSsGAQQBgjcRATFQHk4ATQBpAGMAcg\
BvAHMAbwBmAHQAIABTAG8AZgB0AHcAYQByAGUAIABLAGUAeQAgAFMAdABvAH\
IAYQBnAGUAIABQAHIAbwB2AGkAZABlAHIwggP3BgkqhkiG9w0BBwagggPoMI\
ID5AIBADCCA90GCSqGSIb3DQEHATAcBgoqhkiG9w0BDAEDMA4ECBULm0ouXg\
M1AgIH0ICCA7B/6IFJ1ecfFG7vxVu7rvmhMDur1+rUvanKPr89jZiDkWHUKp\
KYe0hr0y4y2Yqeiu96krgADjh7fkD/+DwFrXae2ClUEfc3iCq9lfQR4aY/2O\
iY2f6mjiPUxrjj61eeOBr1uHKZ7qoqiwn+1Y724DP4VrFxrg+sshTHzBC7zG\
rNR+RPjpfN0CNYVN5yaZBArGkY4AB1knv3KeYNLVXfDadLsRLhihzmK3l0RY\
GoPi8Lw5MfAQ8Rud7+X/tAH6Hj4Gag9oFCx0rbJ/cVdttSJdVOw0QyUlAi9p\
pl/fcLkKyWMerPHP+SNv+Wndx4lfkxicfIwk99oGUIg0O2I9RA8yPgz17LTt\
2vMAlZAoR0yVTOruJ1BYqv+m3bLJfkuUrtT5mzfYRXuKq6W/w680l+an7D5X\
ekTMEaxiQyO12ExsV/Y3ibEk+kdb0zexvOCVI5yXirsYurSvlP/pAxNdsgD5\
4/a8S94t9pErWcjbAgMTHwOhW/DVyVz1nFJ7ivR8vVNafFqSaYG44kMMffsv\
4DCqVlM4KmDUrcOrvzSY9EDhIWZsCemQ5DfWHznn5niVwv0TXY1nX5KhxgYo\
nz1KNE11pvYYAxXp5i/6ru27unIdHmpmG3yZCZUO/PWKq/Tu5tyQf6LlZ/gn\
/BnK5Nr68wGlN7dJgygmHHsISAB8xDMQq0TnOaYhakK/ozU2fCp78CZT2YSB\
XwDnDxmKMybrztFHo1NNOqFLecV/DilevRnhP9oT9Vcmkl1JhyYcS5XtsTAY\
b25ONknC0g7/h1kav1t068AdphjQPj31M5+Ro1UCyYk06oshC+Y0MGZTIB99\
7LMtqfz+oVXTVdcjNQfgAK77eGKlNpPvWqa5qKWnbBt89TL2g/rXoFT+anQr\
SpfCoxw4K8ykOUx4yW08kpPPg/FePN6zJhs7Eu1W8iLV/gsKBnJmnsLk6aNh\
pv+L5RcHv9AcXN3sbbU2JJE9w5PGuIhoCpyQNjzv+wjMW/jbw4fve63J2Gkc\
J3SYoy1C82t5ZORr8P+6q8jpJqDPjwD2LeTmvNKy8UrYThWkRbaUCetuwMlF\
b1XMtzJcLPOs1/stapsf2C70KgvJ+zgGxob7xvpGLyHecn8ZlVer7jNkzbxD\
xUOq2KAMBksmMAg0Tyc44jDOHiinR7sARTUcNncqP5qJ3d7v7DsCCQsiFD1x\
NEyQpgrD5WAhSaKiZxDvvkj/GNNVXpdiFr8RGa/IlD0+JdiZ3ujt45OduOhs\
ySr03eOImlzjA7MB8wBwYFKw4DAhoEFKn9FVak9/MKt1kvn+GIpIdTy/+pBB\
R7oStX6AVc66qjoj9/dgAPJTqLBwICB9A='
# spell-checker:enable


class Test_pyopenssl_signer(unittest.TestCase):

    def test_empty(self):
        with self.assertRaises((KeyError, ValueError)):
            pyopenssl_signer.sign(None, {}, {})

    def test_proper_options_good_key_no_pass(self):
        signer = {
            'key_file_format': 'pkcs12',
            'key_data': b64decode(TESTCERT1)
        }
        signature = {
            'type': 'bare',
            'encoding': 'binary',
            'hash_alg': 'sha256',

        }
        data = "Data for testing signer".encode()
        pyopenssl_signer.sign(data, signature, signer)

    def test_proper_options_good_key_pass(self):
        signer = {
            'key_file_format': 'pkcs12',
            'key_data': b64decode(TESTCERT2),
            'key_file_password': 'password'
        }
        signature = {
            'type': 'bare',
            'encoding': 'binary',
            'hash_alg': 'sha256',

        }
        data = "Data for testing signer".encode()
        pyopenssl_signer.sign(data, signature, signer)

    def test_proper_options_bad_key(self):
        # we're going to assume that we're
        with self.assertRaises(ValueError):
            signer = {
                'key_file_format': 'pkcs12',
                'key_data': "hello there"
            }
            signature = {
                'type': 'bare',
                'encoding': 'binary',
                'hash_alg': 'sha256',

            }
            pyopenssl_signer.sign(None, signature, signer)

    def test_invalid_type(self):
        # we're going to assume that we're
        with self.assertRaises(ValueError):
            signature = {
                'type': 'bad_type',
            }
            pyopenssl_signer.sign(None, signature, {})

    def test_invalid_type_options(self):
        # we're going to assume that we're
        with self.assertRaises(ValueError):
            signature = {
                'type': 'bare',
                'type_options': 'not allowed'
            }
            pyopenssl_signer.sign(None, signature, {})



## @file test_edk2_logging.py
# This contains unit tests for the edk2_logging
##
# Copyright (c) Microsoft Corporation
#
# SPDX-License-Identifier: BSD-2-Clause-Patent
##
import io
import os
import tempfile
import unittest
import logging
from edk2toolext import edk2_logging


class Test_edk2_logging(unittest.TestCase):

    def test_can_create_console_logger(self):
        console_logger = edk2_logging.setup_console_logging(False, False)
        self.assertIsNot(console_logger, None, "We created a console logger")
        edk2_logging.stop_logging(console_logger)

    def test_can_create_txt_logger(self):
        test_dir = tempfile.mkdtemp()
        location, txt_logger = edk2_logging.setup_txt_logger(test_dir, "test_txt")
        logging.info("Testing")
        self.assertTrue(os.path.isfile(location), "We should have created a file")
        self.assertIsNot(txt_logger, None, "We created a txt logger")
        edk2_logging.stop_logging(txt_logger)

    def test_none_to_close(self):
        edk2_logging.stop_logging(None)

    def test_can_close_logger(self):
        test_dir = tempfile.mkdtemp()
        location, txt_logger = edk2_logging.setup_txt_logger(test_dir, "test_close")
        logging.critical("Testing")
        self.assertTrue(os.path.isfile(location), "We should have created a file")
        file = open(location, "r")
        num_lines = len(file.readlines())
        file.close()
        self.assertEqual(num_lines, 1, "We should only have one line")
        edk2_logging.stop_logging(txt_logger)
        logging.critical("Test 2")
        file = open(location, "r")
        num_lines2 = len(file.readlines())
        file.close()
        self.assertEqual(num_lines, num_lines2, "We should only have one line")

    def test_scan_compiler_output_generic(self):
        # Input with compiler errors and warnings
        output_stream = io.StringIO("<source_file>error A1: error 1 details\n"
                                    "<source_file>warning B2: warning 2 details\n"
                                    "<source_file>error C3: error 3 details\n"
                                    "<source_file>warning D4: warning 4 details\n")
        expected_output = [(logging.ERROR, "Compiler #1 from <source_file> error 1 details"),
                           (logging.WARNING, "Compiler #2 from <source_file> warning 2 details"),
                           (logging.ERROR, "Compiler #3 from <source_file> error 3 details"),
                           (logging.WARNING, "Compiler #4 from <source_file> warning 4 details")]
        self.assertEqual(edk2_logging.scan_compiler_output(output_stream), expected_output)

        # Input with no issue (empty string)
        output_stream = io.StringIO("")
        expected_output = []
        self.assertEqual(edk2_logging.scan_compiler_output(output_stream), expected_output)

        # Input with no issue (string has content but not expected to match)
        output_stream = io.StringIO("Some debug message string.")
        expected_output = []
        self.assertEqual(edk2_logging.scan_compiler_output(output_stream), expected_output)

        # Input with no issue (string that mentions an error but does not match)
        output_stream = io.StringIO("An error and warning occurred in x.")
        expected_output = []
        self.assertEqual(edk2_logging.scan_compiler_output(output_stream), expected_output)

        # Input with only compiler warnings
        output_stream = io.StringIO("<source_file.c>warning C8: warning details...\n"
                                    "<source_file.h>warning D10: info about the issue\n")
        expected_output = [(logging.WARNING, "Compiler #8 from <source_file.c> warning details..."),
                           (logging.WARNING, "Compiler #10 from <source_file.h> info about the issue")]
        self.assertEqual(edk2_logging.scan_compiler_output(output_stream), expected_output)

        # Input with only compiler errors
        output_stream = io.StringIO("dir/file.c error T4: uninitialized variable c...\n"
                                    "dir1/dir2/file1.c error B2: duplicate symbol xyz.\n"
                                    "dir1/file_2.h error 5: header file problem")
        expected_output = [(logging.ERROR, "Compiler #4 from dir/file.c uninitialized variable c..."),
                           (logging.ERROR, "Compiler #2 from dir1/dir2/file1.c duplicate symbol xyz."),
                           (logging.ERROR, "Compiler #5 from dir1/file_2.h header file problem")]
        self.assertEqual(edk2_logging.scan_compiler_output(output_stream), expected_output)

        # Input with near matches that should not match
        output_stream = io.StringIO("source.c error A1 error 1 details.\n"
                                    "source warning D6 warning 6 details\n"
                                    "source.obj LNK4: linker 4 details\n"
                                    "script.py 5E: build 5 details\n")
        expected_output = []
        self.assertEqual(edk2_logging.scan_compiler_output(output_stream), expected_output)

        # Test input with different error types
        output_stream = io.StringIO("source.c error A1: error 1 details\n"
                                    "source.c warning B2: warning 2 details\n"
                                    "source.dsc error F3: error 3 details\n"
                                    "source.obj error LNK4: linker 4 details\n"
                                    "script.py error 5E: build 5 details\n")
        expected_output = [(logging.ERROR, "Compiler #1 from source.c error 1 details"),
                           (logging.WARNING, "Compiler #2 from source.c warning 2 details"),
                           (logging.ERROR, "EDK2 #3 from source.dsc error 3 details"),
                           (logging.ERROR, "Linker #4 from source.obj linker 4 details"),
                           (logging.ERROR, "Build.py #5 from script.py build 5 details")]
        self.assertEqual(edk2_logging.scan_compiler_output(output_stream), expected_output)

    def test_scan_compiler_output_vs_actual(self):
        output_stream = io.StringIO("""
            "C:\\Program Files\\Microsoft Visual Studio\\2022\\Enterprise\\VC\\Tools\\MSVC\\14.34.31933\\bin\\Hostx86\\x64\\cl.exe" /Fod:\\a\\1\\s\\Build\\SetupDataPkg\\DEBUG_VS2022\\X64\\SetupDataPkg\\ConfApp\\ConfApp\\OUTPUT\\SvdUsb\\ /showIncludes /nologo /c /WX /GS /W4 /Gs32768 /D UNICODE /O1b2s /GL /Gy /FIAutoGen.h /EHs-c- /GR- /GF /Z7 /Gw -D DISABLE_NEW_DEPRECATED_INTERFACES /Id:\\a\\1\\s\\SetupDataPkg\\ConfApp\\SvdUsb  /Id:\\a\\1\\s\\SetupDataPkg\\ConfApp  /Id:\\a\\1\\s\\Build\\SetupDataPkg\\DEBUG_VS2022\\X64\\SetupDataPkg\\ConfApp\\ConfApp\\DEBUG  /Id:\\a\\1\\s\\MU_BASECORE\\MdePkg  /Id:\\a\\1\\s\\MU_BASECORE\\MdePkg\\Include  /Id:\\a\\1\\s\\MU_BASECORE\\MdePkg\\Test\\UnitTest\\Include  /Id:\\a\\1\\s\\MU_BASECORE\\MdePkg\\Include\\X64  /Id:\\a\\1\\s\\MU_BASECORE\\MdeModulePkg  /Id:\\a\\1\\s\\MU_BASECORE\\MdeModulePkg\\Include  /Id:\\a\\1\\s\\SetupDataPkg  /Id:\\a\\1\\s\\SetupDataPkg\\Include  /Id:\\a\\1\\s\\SetupDataPkg\\Test\\Include  /Id:\\a\\1\\s\\Common\\MU_PLUS\\PcBdsPkg  /Id:\\a\\1\\s\\Common\\MU_PLUS\\PcBdsPkg\\Include  /Id:\\a\\1\\s\\Common\\MU_PLUS\\MsCorePkg  /Id:\\a\\1\\s\\Common\\MU_PLUS\\MsCorePkg\\Include  /Id:\\a\\1\\s\\Common\\MU_PLUS\\XmlSupportPkg  /Id:\\a\\1\\s\\Common\\MU_PLUS\\XmlSupportPkg\\Include  /Id:\\a\\1\\s\\Common\\MU_TIANO_PLUS\\SecurityPkg  /Id:\\a\\1\\s\\Common\\MU_TIANO_PLUS\\SecurityPkg\\Include  /Id:\\a\\1\\s\\MU_BASECORE\\PolicyServicePkg  /Id:\\a\\1\\s\\MU_BASECORE\\PolicyServicePkg\\Include d:\\a\\1\\s\\SetupDataPkg\\ConfApp\\SvdUsb\\SvdUsb.c
            SvdUsb.c
            "C:\\Program Files\\Microsoft Visual Studio\\2022\\Enterprise\\VC\\Tools\\MSVC\\14.34.31933\\bin\\Hostx86\\x64\\lib.exe" /NOLOGO /LTCG /OUT:d:\\a\\1\\s\\Build\\SetupDataPkg\\DEBUG_VS2022\\X64\\SetupDataPkg\\ConfApp\\ConfApp\\OUTPUT\\ConfApp.lib @d:\\a\\1\\s\\Build\\SetupDataPkg\\DEBUG_VS2022\\X64\\SetupDataPkg\\ConfApp\\ConfApp\\OUTPUT\\object_files.lst
            INFO - 	"C:\\Program Files\\Microsoft Visual Studio\\2022\\Enterprise\\VC\\Tools\\MSVC\\14.34.31933\\bin\\Hostx86\\x64\\link.exe" /OUT:d:\\a\\1\\s\\Build\\SetupDataPkg\\DEBUG_VS2022\\X64\\SetupDataPkg\\ConfApp\\ConfApp\\DEBUG\\ConfApp.dll /NOLOGO /NODEFAULTLIB /IGNORE:4001 /IGNORE:4281 /OPT:REF /OPT:ICF=10 /MAP /ALIGN:32 /SECTION:.xdata,D /SECTION:.pdata,D /Machine:X64 /LTCG /DLL /ENTRY:_ModuleEntryPoint /SUBSYSTEM:CONSOLE /SAFESEH:NO /BASE:0 /DRIVER /DEBUG /ALIGN:4096 /DLL /NXCOMPAT   @d:\\a\\1\\s\\Build\\SetupDataPkg\\DEBUG_VS2022\\X64\\SetupDataPkg\\ConfApp\\ConfApp\\OUTPUT\\static_library_files.lst
            UefiApplicationEntryPoint.lib(ApplicationEntryPoint.obj) : error LNK2001: unresolved external symbol __security_check_cookie
            ConfApp.lib(SetupConf.obj) : error LNK2001: unresolved external symbol __report_rangecheckfailure
            d:\\a\\1\\s\\Build\\SetupDataPkg\\DEBUG_VS2022\\X64\\SetupDataPkg\\ConfApp\\ConfApp\\DEBUG\\ConfApp.dll : fatal error LNK1120: 2 unresolved externals
            NMAKE : fatal error U1077: '"C:\\Program Files\\Microsoft Visual Studio\\2022\\Enterprise\\VC\\Tools\\MSVC\\14.34.31933\\bin\\Hostx86\\x64\\link.exe"' : return code '0x460'
            Stop.


            build.py...
            : error 7000: Failed to execute command
                C:\\Program Files\\Microsoft Visual Studio\\2022\\Enterprise\\VC\\Tools\\MSVC\\14.34.31933\\bin\\Hostx86\\x86\\nmake.exe /nologo tbuild [d:\\a\\1\\s\\Build\\SetupDataPkg\\DEBUG_VS2022\\X64\\SetupDataPkg\\ConfApp\\ConfApp]


            build.py...
            : error F002: Failed to build module
                d:\\a\\1\\s\\SetupDataPkg\\ConfApp\\ConfApp.inf [X64, VS2022, DEBUG]
            """)    # noqa: E501

        expected_output = [(logging.ERROR, "Linker #2001 from UefiApplicationEntryPoint.lib(ApplicationEntryPoint.obj) : unresolved external symbol __security_check_cookie"),  # noqa: E501
                           (logging.ERROR, "Linker #2001 from ConfApp.lib(SetupConf.obj) : unresolved external symbol __report_rangecheckfailure"),  # noqa: E501
                           (logging.ERROR, "Linker #1120 from d:\\a\\1\\s\\Build\\SetupDataPkg\\DEBUG_VS2022\\X64\\SetupDataPkg\\ConfApp\\ConfApp\\DEBUG\\ConfApp.dll : fatal 2 unresolved externals"),  # noqa: E501
                           (logging.ERROR, "Compiler #1077 from NMAKE : fatal \'\"C:\\Program Files\\Microsoft Visual Studio\\2022\\Enterprise\\VC\\Tools\\MSVC\\14.34.31933\\bin\\Hostx86\\x64\\link.exe\"\' : return code \'0x460\'"),  # noqa: E501
                           (logging.ERROR, "Compiler #7000 from : Failed to execute command"),  # noqa: E501
                           (logging.ERROR, "EDK2 #002 from : Failed to build module")]
        self.assertEqual(edk2_logging.scan_compiler_output(output_stream), expected_output)

    def test_scan_compiler_output_vs_linker_actual(self):
        output_stream = io.StringIO("""
                copy /y d:/a/1/s/Build/MdeModule/RELEASE_VS2022/IA32/MdeModulePkg/Universal/LegacyRegion2Dxe/LegacyRegion2Dxe/DEBUG/*.map d:/a/1/s/Build/MdeModule/RELEASE_VS2022/IA32/MdeModulePkg/Universal/LegacyRegion2Dxe/LegacyRegion2Dxe/OUTPUT
            SdMmcPciHcPei.lib(SdMmcPciHcPei.obj) : error LNK2001: unresolved external symbol _SafeUint8Add
            d:/a/1/s/Build/MdeModule/RELEASE_VS2022/IA32/MdeModulePkg/Bus/Pci/SdMmcPciHcPei/SdMmcPciHcPei/DEBUG/SdMmcPciHcPei.dll : fatal error LNK1120: 1 unresolved externals
            d:/a/1/s/Build/MdeModule/RELEASE_VS2022/IA32/MdeModulePkg/Universal/LegacyRegion2Dxe/LegacyRegion2Dxe/DEBUG/LegacyRegion2Dxe.map
                    1 file(s) copied.
                copy /y d:/a/1/s/Build/MdeModule/RELEASE_VS2022/IA32/MdeModulePkg/Universal/LegacyRegion2Dxe/LegacyRegion2Dxe/DEBUG/*.pdb d:/a/1/s/Build/MdeModule/RELEASE_VS2022/IA32/MdeModulePkg/Universal/LegacyRegion2Dxe/LegacyRegion2Dxe/OUTPUT
            NMAKE : fatal error U1077: '"C:/Program Files/Microsoft Visual Studio/2022/Enterprise/VC/Tools/MSVC/14.34.31933/bin/Hostx86/x86/link.exe"' : return code '0x460'
            """)    # noqa: E501

        expected_output = [(logging.ERROR, "Linker #2001 from SdMmcPciHcPei.lib(SdMmcPciHcPei.obj) : unresolved external symbol _SafeUint8Add"),  # noqa: E501
                           (logging.ERROR, "Linker #1120 from d:/a/1/s/Build/MdeModule/RELEASE_VS2022/IA32/MdeModulePkg/Bus/Pci/SdMmcPciHcPei/SdMmcPciHcPei/DEBUG/SdMmcPciHcPei.dll : fatal 1 unresolved externals"),  # noqa: E501
                           (logging.ERROR, "Compiler #1077 from NMAKE : fatal \'\"C:/Program Files/Microsoft Visual Studio/2022/Enterprise/VC/Tools/MSVC/14.34.31933/bin/Hostx86/x86/link.exe\"\' : return code \'0x460\'")]  # noqa: E501
        self.assertEqual(edk2_logging.scan_compiler_output(output_stream), expected_output)

    def test_scan_compiler_output_gcc_mixed_actual(self):
        # Test input with all types of issues (errors and warnings)
        output_stream = io.StringIO("""
            "/usr/bin/aarch64-linux-gnu-gcc"   -g -Os -fshort-wchar -fno-builtin -fno-strict-aliasing -Wall -Werror -Wno-array-bounds -include AutoGen.h -fno-common -ffunction-sections -fdata-sections -DSTRING_ARRAY_NAME=BaseLibStrings -g -Os -fshort-wchar -fno-builtin -fno-strict-aliasing -Wall -Werror -Wno-array-bounds -include AutoGen.h -fno-common -mlittle-endian -fno-short-enums -fverbose-asm -funsigned-char -ffunction-sections -fdata-sections -Wno-address -fno-asynchronous-unwind-tables -fno-unwind-tables -fno-pic -fno-pie -ffixed-x18 -mcmodel=small -flto -Wno-unused-but-set-variable -Wno-unused-const-variable -D DISABLE_NEW_DEPRECATED_INTERFACES -mstrict-align -mgeneral-regs-only -c -o /__w/1/s/Build/SetupDataPkg/DEBUG_GCC5/AARCH64/MdePkg/Library/BaseLib/BaseLib/OUTPUT/./FilePaths.obj -I/__w/1/s/MU_BASECORE/MdePkg/Library/BaseLib/AArch64 -I/__w/1/s/MU_BASECORE/MdePkg/Library/BaseLib/Arm -I/__w/1/s/MU_BASECORE/MdePkg/Library/BaseLib -I/__w/1/s/Build/SetupDataPkg/DEBUG_GCC5/AARCH64/MdePkg/Library/BaseLib/BaseLib/DEBUG -I/__w/1/s/MU_BASECORE/MdePkg -I/__w/1/s/MU_BASECORE/MdePkg/Include -I/__w/1/s/MU_BASECORE/MdePkg/Test/UnitTest/Include -I/__w/1/s/MU_BASECORE/MdePkg/Include/AArch64 /__w/1/s/MU_BASECORE/MdePkg/Library/BaseLib/FilePaths.c
            "/usr/bin/aarch64-linux-gnu-gcc"   -g -Os -fshort-wchar -fno-builtin -fno-strict-aliasing -Wall -Werror -Wno-array-bounds -include AutoGen.h -fno-common -ffunction-sections -fdata-sections -DSTRING_ARRAY_NAME=BaseLibStrings -g -Os -fshort-wchar -fno-builtin -fno-strict-aliasing -Wall -Werror -Wno-array-bounds -include AutoGen.h -fno-common -mlittle-endian -fno-short-enums -fverbose-asm -funsigned-char -ffunction-sections -fdata-sections -Wno-address -fno-asynchronous-unwind-tables -fno-unwind-tables -fno-pic -fno-pie -ffixed-x18 -mcmodel=small -flto -Wno-unused-but-set-variable -Wno-unused-const-variable -D DISABLE_NEW_DEPRECATED_INTERFACES -mstrict-align -mgeneral-regs-only -c -o /__w/1/s/Build/SetupDataPkg/DEBUG_GCC5/AARCH64/MdePkg/Library/BaseLib/BaseLib/OUTPUT/./GetPowerOfTwo32.obj -I/__w/1/s/MU_BASECORE/MdePkg/Library/BaseLib/AArch64 -I/__w/1/s/MU_BASECORE/MdePkg/Library/BaseLib/Arm -I/__w/1/s/MU_BASECORE/MdePkg/Library/BaseLib -I/__w/1/s/Build/SetupDataPkg/DEBUG_GCC5/AARCH64/MdePkg/Library/BaseLib/BaseLib/DEBUG -I/__w/1/s/MU_BASECORE/MdePkg -I/__w/1/s/MU_BASECORE/MdePkg/Include -I/__w/1/s/MU_BASECORE/MdePkg/Test/UnitTest/Include -I/__w/1/s/MU_BASECORE/MdePkg/Include/AArch64 /__w/1/s/MU_BASECORE/MdePkg/Library/BaseLib/GetPowerOfTwo32.c
            /__w/1/s/SetupDataPkg/Library/ConfigVariableListLib/ConfigVariableListLib.c:20:1: error: conflicting types for `ConvertVariableListToVariableEntry`; have `EFI_STATUS(const void *, UINTN *, CONFIG_VAR_LIST_ENTRY *)` {aka `long long unsigned int(const void *, long long unsigned int *, CONFIG_VAR_LIST_ENTRY *)`}
            20 | ConvertVariableListToVariableEntry (
                | ^~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            In file included from /__w/1/s/SetupDataPkg/Library/ConfigVariableListLib/ConfigVariableListLib.c:16:
            /__w/1/s/SetupDataPkg/Include/Library/ConfigVariableListLib.h:123:1: note: previous declaration of `ConvertVariableListToVariableEntry` with type `EFI_STATUS(void *, UINTN *, CONFIG_VAR_LIST_ENTRY *)` {aka `long long unsigned int(void *, long long unsigned int *, CONFIG_VAR_LIST_ENTRY *)`}
            123 | ConvertVariableListToVariableEntry (
                | ^~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            "/usr/bin/aarch64-linux-gnu-gcc"   -g -Os -fshort-wchar -fno-builtin -fno-strict-aliasing -Wall -Werror -Wno-array-bounds -include AutoGen.h -fno-common -ffunction-sections -fdata-sections -DSTRING_ARRAY_NAME=BaseLibStrings -g -Os -fshort-wchar -fno-builtin -fno-strict-aliasing -Wall -Werror -Wno-array-bounds -include AutoGen.h -fno-common -mlittle-endian -fno-short-enums -fverbose-asm -funsigned-char -ffunction-sections -fdata-sections -Wno-address -fno-asynchronous-unwind-tables -fno-unwind-tables -fno-pic -fno-pie -ffixed-x18 -mcmodel=small -flto -Wno-unused-but-set-variable -Wno-unused-const-variable -D DISABLE_NEW_DEPRECATED_INTERFACES -mstrict-align -mgeneral-regs-only -c -o /__w/1/s/Build/SetupDataPkg/DEBUG_GCC5/AARCH64/MdePkg/Library/BaseLib/BaseLib/OUTPUT/./GetPowerOfTwo64.obj -I/__w/1/s/MU_BASECORE/MdePkg/Library/BaseLib/AArch64 -I/__w/1/s/MU_BASECORE/MdePkg/Library/BaseLib/Arm -I/__w/1/s/MU_BASECORE/MdePkg/Library/BaseLib -I/__w/1/s/Build/SetupDataPkg/DEBUG_GCC5/AARCH64/MdePkg/Library/BaseLib/BaseLib/DEBUG -I/__w/1/s/MU_BASECORE/MdePkg -I/__w/1/s/MU_BASECORE/MdePkg/Include -I/__w/1/s/MU_BASECORE/MdePkg/Test/UnitTest/Include -I/__w/1/s/MU_BASECORE/MdePkg/Include/AArch64 /__w/1/s/MU_BASECORE/MdePkg/Library/BaseLib/GetPowerOfTwo64.c
            make: *** [GNUmakefile:297: /__w/1/s/Build/SetupDataPkg/DEBUG_GCC5/AARCH64/SetupDataPkg/Library/ConfigVariableListLib/ConfigVariableListLib/OUTPUT/ConfigVariableListLib.obj] Error 1


            build.py...
            : error 7000: Failed to execute command
                make tbuild [/__w/1/s/Build/SetupDataPkg/DEBUG_GCC5/AARCH64/SetupDataPkg/Library/ConfigVariableListLib/ConfigVariableListLib]


            build.py...
            : error F002: Failed to build module
                /__w/1/s/SetupDataPkg/Library/ConfigVariableListLib/ConfigVariableListLib.inf [AARCH64, GCC5, DEBUG]
        """)    # noqa: E501

        expected_output = [(logging.ERROR, "Compiler #error from /__w/1/s/SetupDataPkg/Library/ConfigVariableListLib/ConfigVariableListLib.c conflicting types for `ConvertVariableListToVariableEntry`; have `EFI_STATUS(const void *, UINTN *, CONFIG_VAR_LIST_ENTRY *)` {aka `long long unsigned int(const void *, long long unsigned int *, CONFIG_VAR_LIST_ENTRY *)`}"),  # noqa: E501
                           (logging.ERROR, "Compiler #7000 from : Failed to execute command"),
                           (logging.ERROR, "EDK2 #002 from : Failed to build module")]
        self.assertEqual(edk2_logging.scan_compiler_output(output_stream), expected_output)


# caplog is a pytest fixture that captures log messages
def test_catch_secrets_filter(caplog):
    caplog.set_level(logging.DEBUG)
    edk2_logging.setup_console_logging(logging.DEBUG)

    end_list = [" ", ",", ";", ":", " "]
    start_list = [" ", " ", " ", " ", ":"]

    # Test secret github (valid) 1
    fake_secret = "ghp_aeiou1"
    for start, end in zip(start_list, end_list):
        logging.debug(f"This is a secret{start}{fake_secret}{end}to be caught")

    for (record, start, end) in zip(caplog.records, start_list, end_list):
        assert record.msg == f"This is a secret{start}*******{end}to be caught"
    caplog.clear()

    # Test secret github (valid) 2
    fake_secret = "gho_aeiou1"
    for start, end in zip(start_list, end_list):
        logging.debug(f"This is a secret{start}{fake_secret}{end}to be caught")

    for (record, start, end) in zip(caplog.records, start_list, end_list):
        assert record.msg == f"This is a secret{start}*******{end}to be caught"
    caplog.clear()

    # Test secret github (invalid) 1
    fake_secret = "ghz_aeiou1"
    for start, end in zip(start_list, end_list):
        logging.debug(f"This is a secret{start}{fake_secret}{end}to be caught")

    for (record, start, end) in zip(caplog.records, start_list, end_list):
        assert record.msg == f"This is a secret{start}{fake_secret}{end}to be caught"
    caplog.clear()

    # Test secret nuget (valid) 1
    fake_secret = "345def553aef545fffff42dqa56890493223dfasdfasdf"
    for start, end in zip(start_list, end_list):
        logging.debug(f"This is a secret{start}{fake_secret}{end}to be caught")

    for (record, start, end) in zip(caplog.records, start_list, end_list):
        assert record.msg == f"This is a secret{start}*******{end}to be caught"
    caplog.clear()

    # Test secret nuget (invalid) 1
    fake_secret = "345def553aef545fffff42dqa56890493223dfasdfasd"
    for start, end in zip(start_list, end_list):
        logging.debug(f"This is a secret{start}{fake_secret}{end}to be caught")

    for (record, start, end) in zip(caplog.records, start_list, end_list):
        assert record.msg == f"This is a secret{start}{fake_secret}{end}to be caught"
    caplog.clear()
